@@ -1,5 +1,5 @@
 #  Pyrogram - Telegram MTProto API Client Library for Python
-#  Copyright (C) 2017-2020 Dan <https://github.com/delivrance>
+#  Copyright (C) 2017-present Dan <https://github.com/delivrance>
 #
 #  This file is part of Pyrogram.
 #
@@ -21,6 +21,7 @@ import ipaddress
 import logging
 import socket
 import time
+from typing import Optional
 
 try:
     import socks
@@ -47,9 +48,8 @@ class TCP:
         self.lock = asyncio.Lock()
         self.loop = asyncio.get_event_loop()
 
-        if proxy.get("enabled", False):
-            hostname = proxy.get("hostname", None)
-            port = proxy.get("port", None)
+        if proxy:
+            hostname = proxy.get("hostname")
 
             try:
                 ip_address = ipaddress.ip_address(hostname)
@@ -62,27 +62,35 @@ class TCP:
                     self.socket = socks.socksocket(socket.AF_INET)
 
             self.socket.set_proxy(
-                proxy_type=socks.SOCKS5,
+                proxy_type=getattr(socks, proxy.get("scheme").upper()),
                 addr=hostname,
-                port=port,
+                port=proxy.get("port", None),
                 username=proxy.get("username", None),
                 password=proxy.get("password", None)
             )
 
-            log.info(f"Using proxy {hostname}:{port}")
+            log.info(f"Using proxy {hostname}")
         else:
             self.socket = socks.socksocket(
                 socket.AF_INET6 if ipv6
                 else socket.AF_INET
             )
 
+        self.socket.setblocking(False)
         self.socket.settimeout(TCP.TIMEOUT)
 
+        self.send_queue = asyncio.Queue()
+        self.send_task = None
+
     async def connect(self, address: tuple):
-        self.socket.connect(address)
+        await asyncio.get_event_loop().sock_connect(self.socket, address)
         self.reader, self.writer = await asyncio.open_connection(sock=self.socket)
+        self.send_task = asyncio.create_task(self.send_worker())
+
+    async def close(self):
+        await self.send_queue.put(None)
+        await self.send_task
 
-    def close(self):
         try:
             self.writer.close()
         except AttributeError:
@@ -96,8 +104,16 @@ class TCP:
                 time.sleep(0.001)
                 self.socket.close()
 
-    async def send(self, data: bytes):
-        async with self.lock:
+    async def send(self, data: Optional[bytes]):
+        await self.send_queue.put(data)
+
+    async def send_worker(self):
+        while True:
+            data = await self.send_queue.get()
+
+            if data is None:
+                break
+
             self.writer.write(data)
             await self.writer.drain()
 
